import argparse
from .config import set_config, set_config_manually, print_config
from .access import run, print_access, set_access_manually, check_valid
from .access import load_access


def main():
    '''
    Command Line Interface for Fyers API Helper
    '''

    parser = argparse.ArgumentParser(
        description='CLI for configuration and access operations.'
    )

    subparsers = parser.add_subparsers(
        title='subcommands',
        dest='command',
        help='Choose a command'
    )

    # Config subcommand
    config_parser = subparsers.add_parser('config', help='Open the config')
    config_parser.set_defaults(func=set_config)
    config_parser.add_argument(
        '-p',
        '--print',
        action='store_true',
        dest='print_config',
        help='Print the config'
    )
    config_parser.add_argument(
        '-s',
        '--set',
        nargs=2,
        metavar=('KEY', 'VALUE'),
        dest='set_config',
        help='Set config manually'
    )

    # Access subcommand
    access_parser = subparsers.add_parser(
        'access',
        help='Open the access'
    )
    access_parser.set_defaults(func=run)
    access_parser.add_argument(
        '-v',
        '--is-valid',
        action='store_true',
        dest='is_valid',
        help='Check if the access token is valid'
    )
    access_parser.add_argument(
        '-p',
        '--print',
        action='store_true',
        dest='print_access',
        help='Print the access token'
    )
    access_parser.add_argument(
        '-s',
        '--set',
        nargs=2,
        metavar=('TOKEN', 'AUTODATE'),
        dest='set_access',
        help='Set access token manually'
    )

    args = parser.parse_args()

    if not args.command:
        parser.print_help()
    elif args.command == 'config':
        if args.print_config:
            print_config()
        elif args.set_config:
            set_config_manually(args.set_config[0], args.set_config[1])
        elif args.func:
            args.func()
    elif args.command == 'access':
        if args.is_valid:
            try:
                data = load_access()
                check_valid(data, silent=False)
            except Exception:
                print("Invalid Access Token. Please Generate Token Again")
                exit(1)
        elif args.print_access:
            print_access()
        elif args.set_access:
            set_access_manually(args.set_access[0], args.set_access[1])
        elif args.func:
            args.func()


if __name__ == "__main__":
    main()



from flask import Flask, request, render_template
from waitress import serve
import webbrowser
import json
import os

app = Flask(__name__)
app.secret_key = 'SAMPLE SECRET'

app_id = ''
secret_id = ''
redirect_url = ''

CONFIG_FILE_PATH = os.path.join(os.path.expanduser("~"), ".config.json")


class FyersAPIHelper(Exception):
    pass


@app.route('/', methods=['GET', 'POST'])
def index():
    global app_id, secret_id, redirect_url
    if request.method == 'POST':
        app_id = request.form['app_id']
        secret_id = request.form['secret_id']
        redirect_url = request.form['redirect_url']
        save_config()
        print("Information Saved Successfully .. Press Ctrl-C to continue")
        return "Information Saved Successfully"
    return render_template('config.html')


def save_config():
    data = {
            'APP_ID': app_id,
            'SECRET_ID': secret_id,
            'REDIRECT_URL': redirect_url
            }
    set_data(data)


def load_config():
    # I have no idea why this works with try catch block only.
    try:
        with open(CONFIG_FILE_PATH, 'r') as f:
            data = json.load(f)
    except FileNotFoundError:
        data = {
            'APP_ID': '',
            'SECRET_ID': '',
            'REDIRECT_URL': ''
            }
        set_data(data)
        with open(CONFIG_FILE_PATH, 'r') as f:
            data = json.load(f)
    return data


def get_app_id():
    data = load_config()
    return data["APP_ID"]


def get_secret_id():
    data = load_config()
    return data["SECRET_ID"]


def get_redirect_url():
    data = load_config()
    return data["REDIRECT_URL"]


def set_config():
    webbrowser.open_new('http://localhost:7001')
    serve(app, host='127.0.0.1', port=7001)


def set_config_manually(option, value):
    data = load_config()
    data[option] = value
    set_data(data)


def set_data(data):
    with open(CONFIG_FILE_PATH, 'w+') as f:
        json.dump(data, f)


def print_config():
    try:
        data = load_config()
        print(f'''
APP ID:         {data["APP_ID"]}
SECRET ID:      {data["SECRET_ID"]}
REDIRECT URL:   {data["REDIRECT_URL"]}
            ''')
    except Exception:
        print('Configuration not found. Please set up configuration first.')
        exit(1)


if __name__ == "__main__":
    set_config()



from .config import get_app_id, get_secret_id, get_redirect_url
from flask import Flask, request, render_template
from waitress import serve
import webbrowser
import json
from datetime import datetime
from fyers_apiv3 import fyersModel
import threading
import os

today = datetime.today()

ACCESS_FILE_PATH = os.path.join(os.path.expanduser("~"), ".access.json")

app = Flask(__name__)
app.secret_key = 'SAMPLE SECRET'

access_token = ''
auth_code = ''
appSession = None

try:
    APP_ID = get_app_id()
    SECRET_KEY = get_secret_id()
    REDIRECT_URL = get_redirect_url()
except Exception:
    APP_ID = ""
    SECRET_KEY = ""
    REDIRECT_URL = ""


def get_access(silent=False):
    global APP_ID, REDIRECT_URL, SECRET_KEY, appSession
    grant_type = 'authorization_code'
    response_type = 'code'
    state = 'sample'
    appSession = fyersModel.SessionModel(
            client_id=APP_ID,
            redirect_uri=REDIRECT_URL,
            secret_key=SECRET_KEY,
            response_type=response_type,
            state=state,
            grant_type=grant_type)
    generateTokenUrl = appSession.generate_authcode()
    if not silent:
        webbrowser.open(generateTokenUrl, new=1)
    else:
        print(generateTokenUrl)


class FyersAPIHelper(Exception):
    pass


@app.route('/', methods=['GET', 'POST'])
def index():
    global access_token, today, auth_code, appSession
    auth_code = request.args.get('auth_code', '')
    if request.method == 'POST':
        auth_code = request.form['auth_code']
        today = request.form['date']
        appSession.set_token(auth_code)
        response = appSession.generate_token()
        access_token = response['access_token']
        save_access()
        print("Access Token saved Successfully .. Press Ctrl-C to continue")
        return "Access Token saved Successfully"
    return render_template(
            'access.html',
            auth_code=auth_code,
            today=today.strftime('%d-%m-%Y'))


def save_access():
    data = {
            'ACCESS_TOKEN': access_token,
            'DATE': today,
            }
    set_data(data)


def load_access():
    try:
        with open(ACCESS_FILE_PATH, 'r') as f:
            data = json.load(f)
    except FileNotFoundError:
        data = {
            'ACCESS_TOKEN': '',
            'DATE': '',
            }
        set_data(data)
        with open(ACCESS_FILE_PATH, 'r') as f:
            data = json.load(f)
    return data


def set_access():
    serve(app, host='127.0.0.1', port=7000)


def set_access_manually(option, value):
    data = load_access()
    data[option] = value
    set_data(data)


def set_data(data):
    with open(ACCESS_FILE_PATH, 'w+') as f:
        json.dump(data, f)


def print_access():
    data = load_access()
    print(f'''
ACCESS TOKEN:   {data["ACCESS_TOKEN"]}
DATE:           {data["DATE"]}
          ''')


def check_valid(data, silent=True):
    if data["DATE"] == today.strftime('%d-%m-%Y'):
        if not silent:
            print("Access Token Valid")
        return True
    else:
        if not silent:
            print("Access Token Invalid")
        return False


def get_access_token():
    data = load_access()
    valid = check_valid(data)
    return data["ACCESS_TOKEN"] if valid else None


def run():
    thread_get_access_token = threading.Thread(target=get_access)
    thread_get_access_token.start()
    set_access()


if __name__ == "__main__":
    run()
